@@ -21,7 +21,9 @@ def _convert_totuple(tok):
     is_punct = re.compile("[{}]+$".format(re.escape(string.punctuation)))
     is_digit = re.compile("[\d{}]+$".format(re.escape(string.punctuation)))
     for i in range(0,len(tok)):
-        token_list = [x.lower() for x in list(tok.values())[i]['token']]
+        token_list = list(tok.values())[i]['token']
+        ws_list = list(tok.values())[i]['whitespace']
+        token_list = list(map(''.join, zip(token_list, ws_list)))
         iob_list = list(tok.values())[i]['ent_iob']
         iob_list = [x.replace('IS_DIGIT','B') for x in iob_list]
         iob_list = [x.replace('IS_ALPHA','I') for x in iob_list]
@@ -35,7 +37,6 @@ def _convert_totuple(tok):
         token_tuple.append(list(zip(token_list, tag_list, iob_ent)))
     return(token_tuple)
 
-
 def _groupby_consecutive(lst):
     """
     Convenience function for grouping consecutive items in a list.
@@ -52,7 +53,9 @@ def _merge_tags(tok):
     
     :param tok: a tokens tuple object
     """
-    p = re.compile('[a-z]+')
+    tok = [[(word.lower(), tag, ds) for word, tag, ds in element] for element in tok]
+    tok = [[(word.strip(), tag, ds) for word, tag, ds in element] for element in tok]
+    p = re.compile('[a-z]')
     phrase_list = []
     for i in range(0,len(tok)):
         # filter out strings that don't contain at least one alphabetic character
@@ -77,7 +80,7 @@ def _merge_tags(tok):
                 original_label = subtree.label()
                 original_string = " ".join([token for token, pos in subtree.leaves()])
             else:
-                original_label = 'O'
+                original_label = 'Untagged'
                 original_string = subtree[0]
             agg_tokens.append((original_string, original_label))
         phrase_list.append(agg_tokens)
@@ -89,11 +92,13 @@ def _merge_ds(tok):
     
     :param tok: a tokens tuple object
     """
-    p = re.compile('[^a-z]+')
+    tok = [[(word.lower(), tag, ds) for word, tag, ds in element] for element in tok]
+    tok = [[(word.strip(), tag, ds) for word, tag, ds in element] for element in tok]
+    p = re.compile('[a-z]')
     phrase_list = []
     for i in range(0,len(tok)):
         # filter out strings that don't contain at least one alphabetic character
-        tpf = [x for x in tok[i] if not (x[2] == 'O-' and p.search(x[0]))]
+        tpf = [x for x in tok[i] if not (x[2] == 'O-' and bool(p.search(x[0])) == False)]
         ne_tree = _conlltags2tree(tpf)
         agg_tokens = []
         for subtree in ne_tree:
@@ -101,7 +106,7 @@ def _merge_ds(tok):
                 original_label = subtree.label()
                 original_string = " ".join([token for token, pos in subtree.leaves()])
             else:
-                original_label = 'O'
+                original_label = 'Untagged'
                 original_string = subtree[0]
             agg_tokens.append((original_string, original_label))
         phrase_list.append(agg_tokens)
@@ -208,6 +213,10 @@ def _log_like(n_target, n_reference, total_target, total_reference, correct=Fals
     L1 = 0 if n_a == 0 else n_a*np.log(n_a/expected_a)
     L2 = 0 if n_b == 0 else n_b*np.log(n_b/expected_b)
     likelihood = 2*(L1 + L2)
+    if n_target/total_target > n_reference/total_reference:
+        likelihood = likelihood
+    else:
+        likelihood = -likelihood
     return(likelihood)
 
 
